@@ -4,7 +4,7 @@ use warnings;
 package App::Spec;
 use 5.010;
 
-our $VERSION = '0.002_001'; # TRIAL VERSION
+our $VERSION = '0.003_001'; # TRIAL VERSION
 
 use App::Spec::Command;
 use App::Spec::Option;
@@ -29,6 +29,16 @@ has description => ( is => 'rw' );
 my $DATA = do { local $/; <DATA> };
 my $default_spec;
 
+sub build {
+    my ($class, $spec) = @_;
+    for (@{ $spec->{options} || [] }, @{ $spec->{parameters} || [] }) {
+        $_ = { spec => $_ } unless ref $_;
+    }
+    $_ = App::Spec::Option->build($_) for @{ $spec->{options} || [] };
+    $_ = App::Spec::Parameter->build($_) for @{ $spec->{parameters} || [] };
+    my $self = $class->new($spec);
+}
+
 sub _read_default_spec {
     $default_spec ||= YAML::XS::Load($DATA);
     return dclone $default_spec;
@@ -51,25 +61,7 @@ sub read {
         die "No filename given";
     }
 
-    my $spec;
-    if (ref $file eq 'GLOB') {
-        my $data = do { local $/; <$file> };
-        $spec = eval { YAML::XS::Load($data) };
-    }
-    elsif (not ref $file) {
-        $spec = eval { YAML::XS::LoadFile($file) };
-    }
-    elsif (ref $file eq 'SCALAR') {
-        my $data = $$file;
-        $spec = eval { YAML::XS::Load($data) };
-    }
-    elsif (ref $file eq 'HASH') {
-        $spec = $file;
-    }
-
-    unless ($spec) {
-        die "Error reading '$file': $@";
-    }
+    my $spec = $class->load_data($file);
 
     my $has_subcommands = $spec->{subcommands} ? 1 : 0;
     my $default;
@@ -78,7 +70,11 @@ sub read {
 
         for my $opt (@{ $default->{options} }) {
             my $name = $opt->{name};
-            unless (any { $_->{name} eq $name } @{ $spec->{options} }) {
+            # TODO
+            # this should be moved somewhere else since the name might not
+            # be parsed from dsl yet
+            no warnings 'uninitialized';
+            unless (any { (ref $_ ? $_->{name} : $_) eq $name } @{ $spec->{options} }) {
                 push @{ $spec->{options} }, $opt;
             }
         }
@@ -106,25 +102,35 @@ sub read {
         }
     }
 
-    my $self = $class->new({
-        name => $spec->{name},
-        appspec => $spec->{appspec},
-        class => $spec->{class},
-        title => $spec->{title},
-        markup => $spec->{markup},
-        options => [map {
-            App::Spec::Option->build($_)
-        } @{ $spec->{options} || [] }],
-        parameters => [map {
-            App::Spec::Parameter->build($_)
-        } @{ $spec->{parameters} || [] }],
-        subcommands => $commands,
-        abstract => $spec->{abstract},
-        description => $spec->{description},
-    });
+    $spec->{subcommands} = $commands;
+    my $self = $class->build($spec);
     return $self;
 }
 
+sub load_data {
+    my ($class, $file) = @_;
+    my $spec;
+    if (ref $file eq 'GLOB') {
+        my $data = do { local $/; <$file> };
+        $spec = eval { YAML::XS::Load($data) };
+    }
+    elsif (not ref $file) {
+        $spec = eval { YAML::XS::LoadFile($file) };
+    }
+    elsif (ref $file eq 'SCALAR') {
+        my $data = $$file;
+        $spec = eval { YAML::XS::Load($data) };
+    }
+    elsif (ref $file eq 'HASH') {
+        $spec = $file;
+    }
+
+    unless ($spec) {
+        die "Error reading '$file': $@";
+    }
+    return $spec;
+}
+
 sub _add_subcommands {
     my ($self, $commands1, $commands2, $ref) = @_;
     for my $name (keys %{ $commands2 || {} }) {
@@ -190,7 +196,23 @@ EOM
         }
         $usage .= " $usage_string";
         $body .= "$header\n";
-        for my $name (sort keys %$subcmds) {
+
+        my %keys;
+        @keys{ keys %$subcmds } = ();
+        my @keys;
+        if (@$cmds) {
+            @keys = sort keys %keys;
+        }
+        else {
+            for my $key (qw/ help _meta /) {
+                if (exists $keys{ $key }) {
+                    push @keys, $key;
+                    delete $keys{ $key };
+                }
+            }
+            unshift @keys, sort keys %keys;
+        }
+        for my $name (@keys) {
             my $cmd_spec = $subcmds->{ $name };
             my $summary = $cmd_spec->summary;
             push @table, [$name, $summary];
@@ -391,13 +413,8 @@ sub make_getopt {
         }
         $specs->{ $name } = $opt;
         if ($opt->multiple) {
-            if ($opt->multiple) {
-                if ($opt->type eq 'flag') {
-                    $spec .= '+';
-                }
-                else {
-                    $spec .= '@';
-                }
+            if ($opt->type eq 'flag') {
+                $spec .= '+';
             }
             else {
                 $result->{ $name } = [];
@@ -426,6 +443,8 @@ structure in perl.
 The L<App::Spec::Run> module is the framework which will run the actual
 app.
 
+Have a look at the L<App::Spec::Tutorial> for how to write an app.
+
 In the examples directory you will find the app C<myapp> which is supposed
 to demonstrate everything that App::Spec supports right now.
 
@@ -438,15 +457,18 @@ Your script:
     $run->run;
 
     # this is equivalent to
-    #my $run = Your::App->new({
+    #my $run = App::Spec::Run->new(
     #    spec => $spec,
-    #});
+    #    cmd => Your::App->new(
+    #        spec => $spec,
+    #    ),
+    #);
     #$run->run;
 
 Your App class:
 
     package Your::App;
-    use base 'App::Spec::Run';
+    use base 'App::Spec::Run::Cmd';
 
     sub command1 {
         my ($self) = @_;
@@ -464,6 +486,18 @@ Your App class:
 
     my $spec = App::Spec->read("/path/to/myapp-spec.yaml");
 
+=item load_data
+
+Takes a file, hashref or glob and returns generated appspec hashref
+
+    my $hash = $class->load_data($file);
+
+=item build
+
+Builds objects out of the hashref
+
+    my $appspec = App::Spec->build($hashref);
+
 =item runner
 
 Returns an instance of the your app class
@@ -537,22 +571,27 @@ subcommands:
         options:
         -   name: all
             type: flag
-    _complete:
-        summary: Generate self completion
-        op: cmd_self_completion
-        options:
-            -   name: name
-                description: name of the program
-            -   name: zsh
-                description: for zsh
-                type: flag
-            -   name: bash
-                description: for bash
-                type: flag
-#            -   name: without-description
-#                type: flag
-#                default: false
-#                description: generate without description
-    _pod:
-        summary: Generate self pod
-        op: cmd_self_pod
+    _meta:
+        summary: Information and utilities for this app
+        subcommands:
+            completion:
+                summary: Shell completion functions
+                subcommands:
+                    generate:
+                        summary: Generate self completion
+                        op: cmd_self_completion
+                        options:
+                            -   name: name
+                                description: name of the program (optional, override name in spec)
+                            -   name: zsh
+                                description: for zsh
+                                type: flag
+                            -   name: bash
+                                description: for bash
+                                type: flag
+            pod:
+                summary: Pod documentation
+                subcommands:
+                    generate:
+                        summary: Generate self pod
+                        op: cmd_self_pod
