@@ -2,7 +2,7 @@ use strict;
 use warnings;
 package App::Spec::Pod;
 
-our $VERSION = '0.002_001'; # TRIAL VERSION
+our $VERSION = '0.003_001'; # TRIAL VERSION
 
 use Text::Table;
 
@@ -70,7 +70,7 @@ sub subcommand_pod {
         @keys = sort keys %keys;
     }
     else {
-        for my $key (qw/ help _complete _pod /) {
+        for my $key (qw/ help _meta /) {
             if (exists $keys{ $key }) {
                 push @keys, $key;
                 delete $keys{ $key };
